/***
*** internals.h
***
***  General description of this file:
***  	Driver structure and macro definitions for General Standards 16AI32
***  	32 channel PCI/PMC A/D board. 
***
***  Author:
***
***  Support:
***
***
***/
#ifndef INTERNALS_H
#define INTERNALS_H
/*****************************************************************************/
/******                                                                 ******/
/******                         INCLUDE FILES                           ******/
/******                                                                 ******/
/*****************************************************************************/

/*****************************************************************************/
/******                                                                 ******/
/******                          LOCAL DEFINE                           ******/
/******                                                                 ******/
/*****************************************************************************/

/* module name(s) */
#define GSC_NAME      "GSC16AI32" /* Must match name in makefile */
#define DRIVER_VERSION  "1.0"
// 1.1	Fixed a DMA read bug for low data rates. Updated GSC_NAME macro.


#define GSC_SUBVENDOR PCI_VENDOR_ID_PLX
#define GSE_SUBDEVICE16 0x3101
#define VENDOR_ID_SIZE	2


#define MAX_BOARDS 10 /* maximum number of boards that driver supports. Arbitrarily chosen,
change if you can actually support more. */

// PCI BAR Space information

#define GROUP_0 0
#define GROUP_1 1

// interrupt event condition codes
#define GROUP_0_EVENT                       0x00
#define GROUP_1_EVENT                       0x10

#define EVENT_INIT_COMPLETE                 GROUP_0_EVENT+ICR_GROUP_0_IDLE
#define EVENT_AUTOCAL_COMPLETE              GROUP_0_EVENT+ICR_GROUP_0_AUTOCAL_COMPLETE
#define EVENT_INPUT_SCAN_INIT               GROUP_0_EVENT+ICR_GROUP_0_INPUT_SCAN_INIT
#define EVENT_INPUT_SCAN_COMPLETE           GROUP_0_EVENT+ICR_GROUP_0_INPUT_SCAN_COMPLETE

#define EVENT_BUFFER_THRES_LOW_TO_HIGH      GROUP_1_EVENT+ICR_GROUP_1_LO_HI
#define EVENT_BUFFER_THRES_HIGH_TO_LOW      GROUP_1_EVENT+ICR_GROUP_1_HI_LO
#define EVENT_DMA_PENDING                   EVENT_BUFFER_THRES_HIGH_TO_LOW+1
#define EVENT_LAST_IRQ_EVENT                EVENT_DMA_PENDING+1   /* for array allocation */

/* timing and delay stuff */
#define INIT_TIMEOUT_MS       	2000 	    /* initialization timeout */
#define AUTOCAL_TIMEOUT_MS    	5000  	    /* auto calibration timeout */
#define CH_READY_TIMEOUT_MS   	500   	    /* timeout after channel setup operations */
#define MSECS_TO_SLEEP(MS)      ((MS * HZ) / 1000)

/* additional, internally used interrupt event codes */
#define NO_EVENT      	      	(-1)

/* DMA buffer size  */
#define DMA_ORDER     	      	4 /* (64 kBytes on Intel) */
//#define DMA_ORDER     	      	5 /* (128 kBytes on Intel) */

#define	PAGE_SIZE								4096
#define DMA_SIZE      	      	(PAGE_SIZE << DMA_ORDER)
#define DMA_SAMPLES   	      	(DMA_SIZE / sizeof(unsigned long))
#define HW_BUFFER_SIZE 1024*256 /* 256K */

//==============================================================================
// Board Register Addresses
//==============================================================================
// Board Register Addresses 
#define     BCR               0x00
#define     ICR               0x04
#define     IN_DATA_BUFF      0x08
#define     IN_DATA_CNTRL     0x0C
#define     RATE_A            0x10
#define     RATE_B            0x14
#define     BUFF_SIZE					0x18
#define     NB_SAMPLES				0x1C
#define     SCAN_CNTRL        0x20
#define     ACTIVE_CH_ASSIGN  0x24
#define     BOARD_CONF        0x28
#define     AUTOCAL           0x2C
#define		  UVAL							0x30
#define		  AUX SYNC					0x34
#define		  SCAN_MARKER_U			0x38
#define		  SCAN_MARKER_L			0x3C
#define			TIME_TAG_CONF			0x50
#define			ACTIVE_CH_MASK		0x54
#define			TIME_TAG_COUNT_L	0x58
#define			TIME_TAG_COUNT_U	0x5C
#define			TIME_TAG_DIV			0x60
#define			NB_SAMPLES_AFTER_TRIG	0x64
#define			CH0_THRES_REF			0x80

// 32 bits address offset registers
#define BOARD_CTRL_REG			0
#define INT_CTRL_REG	1
#define INPUT_DATA_BUFF_REG		2
#define INPUT_BUFF_CTRL_REG		3
#define RATE_A_GENERATOR_REG	4
#define RATE_B_GENERATOR_REG	5
#define BUFFER_SIZE_REG			6
#define NB_SAMPLES_REG			7
#define SCAN_SYNCH_CTRL_REG		8
#define ACTIVE_CH_ASSIGN_REG			9
#define FIRMWARE_REV_REG		10
#define AUTOCAL_VALUES_REG		11
#define AUX_R_W_REG             12
#define AUX_SYNCH_IO_CTRL       13
#define SCAN_MARK_FIRST_VAL     14
#define SCAN_MARK_SECOND_VAL    15
#define	TIME_TAG_CONF_REG			20
#define	ACTIVE_CH_MASK_REG		21
#define	TIME_TAG_COUNT_L_REG	22
#define	TIME_TAG_COUNT_U_REG	23
#define	TIME_TAG_DIV_REG			24
#define	NB_SAMPLES_AFTER_TRIG_REG	25
#define	CH0_THRES_REF_REG			32

/* ---- masks for the 'local' Board Control Register (32-bit addr 0). ---- */
/* ---- see GSC documentation for info ---- */

// analog input mode

#define BCR_DIFFERENTIAL 0
#define BCR_SINGLE_ENDED 1
#define BCR_ZERO_TEST 2
#define BCR_VREF_TEST 3

#define BCR_AIM_SHIFT                        0
#define BCR_AIM_MASK                         0x0007

// analog input range

#define BCR_RANGE_SHIFT                      4
#define BCR_RANGE_MASK                       (3<<BCR_RANGE_SHIFT)

#define BCR_RANGE_2_5						(0)
#define BCR_RANGE_5	  					    (1)
#define BCR_RANGE_10 						(2)

// unipolar inputs - SSA board only.

#define BCR_UNIPOLAR_INPUTS_SSA             (1<<3)

// offset binary mode 

#define BCR_OFFSET_BINARY_SHIFT              6
#define BCR_FORMAT_MASK                      (1<<BCR_OFFSET_BINARY_SHIFT)

// External synch

#define BCR_ENABLE_EXTERN_SYNC               (1<<7)

// Differential processing modes. SSA boards only.  Choices of:
// - Single ended.
// - Pseudo-differential.
// - Full differential.

#define BCR_DIFF_PROC_SHIFT_SSA             8
#define BCR_DIFF_PROC_MASK_SSA              (3<<BCR_DIFF_PROC_SHIFT_SSA)

#define BCR_DIFF_PROC_SE_SSA                0
#define BCR_DIFF_PROC_PSEUDO_DIFF_SSA       1
#define BCR_DIFF_PROC_FULL_DIFF_SSA         2

// start single scan

#define BCR_INPUT_SYNCH_START				(1<<12)

// init autocalibration

#define BCR_AUTOCAL                         (1<<13)

// autocalibration test passed flag

#define BCR_AUTOCAL_PASS                    (1<<14)

// initialize the hardware

#define BCR_INITIALIZE                      (1<<15)

// Buffer underflow flag

#define BCR_BUFFER_UNDERFLOW_SSA            (1<<16)

// Buffer overflow flag

#define BCR_BUFFER_OVERFLOW_SSA             (1<<17)

#define BCR_ENABLE_TIME_TAG									(1<<20)

// Enables scan marking.  Refer to hardware manual.

#define ENABLE_SCAN_MARKING_SSA             (1<<18)

// Disables demand-mode DMA

#define DISABLE_DEMAND_MODE_SSA             (1<<19)
/* ---- masks for the 'local' Interrupt Control Register (32-bit addr 1). ---- */
/* ---- see GSC documentation for info ---- */

#define ICR_GROUP_0_MASK        0x07
#define ICR_GROUP_0_SHIFT       0
#define ICR_GROUP_0_REQUEST     (1<<3)

#define ICR_GROUP_0_IDLE 0
#define ICR_GROUP_0_AUTOCAL_COMPLETE    1
#define ICR_GROUP_0_INPUT_SCAN_INIT     2
#define ICR_GROUP_0_INPUT_SCAN_COMPLETE 3
#define ICR_GROUP_0_TRIGGER_BURST_INITIALIZED 4
#define ICR_GROUP_0_TRIGGER_BURST_COMPLETED 5

#define ICR_GROUP_1_MASK        0x07
#define ICR_GROUP_1_SHIFT       4
#define ICR_GROUP_1_REQUEST     (1<<7)

#define ICR_GROUP_1_IDLE    0
#define ICR_GROUP_1_LO_HI   1
#define ICR_GROUP_1_HI_LO   2

/* ---- masks for the 'local' Input Data Register (32-bit addr 2). ---- */
/* ---- see GSC documentation for info ---- */

#define VALID_DATA_MASK     0xFF
#define CHANNEL_0_FLAG      (1<<16)

// SSA specific data masks for "packed" data.

#define SCAN_MARKER_DEFAULT_SSA 0x00
#define LO_DATA_MASK_SSA        0xFF
#define HI_DATA_SHIFT_SSA       16
#define HI_DATA_MASK_SSA        (LO_DATA_MASK_SSA<<HI_DATA_SHIFT_SSA)
#define PACKED_DATA_MASK_SSA    0xFFFF

/* ---- masks for the 'local' Input Data Control Registers (32-bit addr 3). ---- */
/* ---- see GSC documentation for info ---- */

#define IDC_INPUT_CLEAR_BUFFER  (1<<16)
#define IDC_THRESHOLD_FLAG      (1<<17)
#define IDC_INPUT_BUFFER_SIZE   0xFFFF 
#define IDC_THRESHOLD_MASK      IDC_INPUT_BUFFER_SIZE

// the maximum size for the hardware buffer.  Used for error checking.
#define MAX_BUFFER_SIZE 512*1024

// these for the SS boards only 

#define IDC_DISABLE_BUFFER_SS               (1<<18)
#define IDC_TERMINATE_DMA_ALMOST_EMPTY_SS   (1<<19)

// These for the SSA board only

#define IDC_THRESHOLD_4X_SSA        (1<<19)

/* ---- masks for the 'local' Rate A Generator Register (32-bit addr 4). ---- */
/* ---- masks for the 'local' Rate B Generator Register (32-bit addr 5). ---- */
/* ---- see GSC documentation for info ---- */

#define RGR_NRATE_MASK (0xFFFF)
#define RGR_GEN_DISABLE (1<<16)

/* ---- masks for the 'local' Buffer Size (32-bit addr 6). ---- */
/* ---- see GSC documentation for info ---- */

/* no masks required... */

/* ---- masks for the 'local' Scan and Synch Control Register(32-bit addr 8). ---- */
/* ---- see GSC documentation for info ---- */

// Scan size defines

#define SSCR_1_CHAN_SIZE 0
#define SSCR_2_CHAN_SIZE 1
#define SSCR_4_CHAN_SIZE 2
#define SSCR_8_CHAN_SIZE 3
#define SSCR_16_CHAN_SIZE 4
#define SSCR_32_CHAN_SIZE 5
#define SSCR_64_CHAN_SIZE 6

#define SSCR_CHAN_SIZE_MASK 0x07

// analog inputs scan clock

#define SSCR_CLOCK_SHIFT 3
#define SSCR_SCAN_CLOCK_MASK (3<<SSCR_CLOCK_SHIFT)

#define SSCR_INTERNAL_RATE_A 0
#define SSCR_INTERNAL_RATE_B (1)
#define SSCR_EXTERNAL_SYNCH  (2)
#define SSCR_BCR_INPUT_SYNCH (3)

// single channel defines

#define SSCR_SINGLE_CHANNEL_SHIFT (12)
#define SSCR_SINGLE_CHANNEL_MASK (0x3F<<12)

// Rate "B" clock source

#define SSCR_RATE_B_CLOCK_SOURCE_SHIFT (10)
#define SSCR_RATE_B_CLOCK_SOURCE_MASK (1<<SSCR_RATE_B_CLOCK_SOURCE_SHIFT)

#define SSCR_MASTER_CLOCK 0
#define SSCR_RATE_A_GENERATOR (1)

/* ---- Auxiliary Sync I/O Control ---- */

#define ASC_MODE_INACTIVE       0
#define ASC_MODE_ACTIVE_IN      1
#define ASC_MODE_ACTIVE_OUT     2

#define ASC_MODE_MASK 0x03

#define ASC_CONTROL_MODE_0_SHIFT 0


#define ASC_MODE_INACTIVE_0     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_0_SHIFT)
#define ASC_MODE_ACTIVE_IN_0    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_0_SHIFT)
#define ASC_MODE_ACTIVE_OUT_0   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_0_SHIFT)

#define ASC_CONTROL_MODE_1_SHIFT 2

#define ASC_MODE_INACTIVE_1     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_1_SHIFT)
#define ASC_MODE_ACTIVE_IN_1    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_1_SHIFT)
#define ASC_MODE_ACTIVE_OUT_1   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_1_SHIFT)

#define ASC_CONTROL_MODE_2_SHIFT 4

#define ASC_MODE_INACTIVE_2     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_2_SHIFT)
#define ASC_MODE_ACTIVE_IN_2    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_2_SHIFT)
#define ASC_MODE_ACTIVE_OUT_2   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_2_SHIFT)

#define ASC_CONTROL_MODE_3_SHIFT 6

#define ASC_MODE_INACTIVE_3     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_3_SHIFT)
#define ASC_MODE_ACTIVE_IN_3    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_3_SHIFT)
#define ASC_MODE_ACTIVE_OUT_3   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_3_SHIFT)

#define ASC_INVERT_INPUTS       (1<<8)
#define ASC_INVERT_OUTPUTS      (1<<9)
#define ASC_NOISE_SUPPRESS      (1<<10)

/* ---- TTC ---- */
#define TTC_SCAN_CLOCK_MASK 0x0003
#define TTC_SCAN_CLOCK_SHIFT 0
#define TTC_ENABLE_SAMPLING 4

/* ---- DMA command/status register bits for channel 0 ---- */
/* ---- see PLX documentation for info ---- */

#define DMA0_ENABLE			    0x0001
#define DMA0_START			    0x0002
#define DMA0_ABORT			    0x0004
#define DMA0_CLR_INT		    0x0008
#define DMA0_DONE			    0x0010

/* ---- input buffer size ---- */

#define INPUT_BUFFER_SIZE (64*1024) 

/*****************************************************************************/
/******                                                                 ******/
/******                          LOCAL MACRO                            ******/
/******                                                                 ******/
/*****************************************************************************/
/* Macros to make register access easier */
/* Mapping Local Configuration Registers */
#define PciLocRange0(card)      ((card)->runtime_addr + PCI_TO_LOC_ADDR_0_RNG)
#define PciLocRemap0(card)      ((card)->runtime_addr + LOC_BASE_ADDR_REMAP_0)
#define ModeArb(card)           ((card)->runtime_addr + MODE_ARBITRATION)
#define EndianDescr(card)       ((card)->runtime_addr + BIG_LITTLE_ENDIAN_DESC)
#define PciLERomRange(card)     ((card)->runtime_addr + PCI_TO_LOC_ROM_RNG)
#define PciLERomRemap(card)     ((card)->runtime_addr + LOC_BASE_ADDR_REMAP_EXP_ROM)
#define PciLBRegDescr0(card)    ((card)->runtime_addr + BUS_REG_DESC_0_FOR_PCI_LOC)
#define LocPciRange(card)       ((card)->runtime_addr + DIR_MASTER_TO_PCI_RNG)
#define LocPciMemBase(card)     ((card)->runtime_addr + LOC_ADDR_FOR_DIR_MASTER_MEM)
#define LocPciIOBase(card)      ((card)->runtime_addr + LOC_ADDR_FOR_DIR_MASTER_IO)
#define LocPciRemap(card)       ((card)->runtime_addr + PCI_ADDR_REMAP_DIR_MASTER)
#define LocPciConfig(card)      ((card)->runtime_addr + PCI_CFG_ADDR_DIR_MASTER_IO)
#define PciLocRange1(card)      ((card)->runtime_addr + PCI_TO_LOC_ADDR_1_RNG)
#define PciLocRemap1(card)      ((card)->runtime_addr + LOC_BASE_ADDR_REMAP_1)
#define PciLBRegDescr1(card)    ((card)->runtime_addr + BUS_REG_DESC_1_FOR_PCI_LOC)

/* Mapping Runtime Registers */
#define Mailbox0(card)          ((card)->runtime_addr + MAILBOX_REGISTER_0)
#define Mailbox1(card)          ((card)->runtime_addr + MAILBOX_REGISTER_1)
#define Mailbox2(card)          ((card)->runtime_addr + MAILBOX_REGISTER_2)
#define Mailbox3(card)          ((card)->runtime_addr + MAILBOX_REGISTER_3)
#define Mailbox4(card)          ((card)->runtime_addr + MAILBOX_REGISTER_4)
#define Mailbox5(card)          ((card)->runtime_addr + MAILBOX_REGISTER_5)
#define Mailbox6(card)          ((card)->runtime_addr + MAILBOX_REGISTER_6)
#define Mailbox7(card)          ((card)->runtime_addr + MAILBOX_REGISTER_7)
#define PciLocDoorBell(card)    ((card)->runtime_addr + PCI_TO_LOC_DOORBELL)
#define LocPciDoorBell(card)    ((card)->runtime_addr + LOC_TO_PCI_DOORBELL)

/* Mapping interrupt control/status reg  */
#define IntCntrlStat(card)      ((card)->runtime_addr + INT_CTRL_STATUS)
#define RunTimeCntrl(card)      ((card)->runtime_addr + PROM_CTRL_CMD_CODES_CTRL)
#define DevVenIDc(card)         ((card)->runtime_addr + DEVICE_ID_VENDOR_ID)
#define RevID(card)             ((card)->runtime_addr + REVISION_ID)
#define MailboxReg0(card)       ((card)->runtime_addr + MAILBOX_REG_0)
#define MailboxReg1(card)       ((card)->runtime_addr + MAILBOX_REG_1)

/* Mapping DMA registers */
#define DMAMode0(card)          ((card)->runtime_addr + DMA_CH_0_MODE)
#define DMAPCIAddr0(card)       ((card)->runtime_addr + DMA_CH_0_PCI_ADDR)
#define DMALocalAddr0(card)     ((card)->runtime_addr + DMA_CH_0_LOCAL_ADDR)
#define DMAByteCnt0(card)       ((card)->runtime_addr + DMA_CH_0_TRANS_BYTE_CNT)
#define DMADescrPtr0(card)      ((card)->runtime_addr + DMA_CH_0_DESC_PTR)
#define DMAMode1(card)          ((card)->runtime_addr + DMA_CH_1_MODE)
#define DMAPCIAddr1(card)       ((card)->runtime_addr + DMA_CH_1_PCI_ADDR)
#define DMALocalAddr1(card)     ((card)->runtime_addr + DMA_CH_1_LOCAL_ADDR)
#define DMAByteCnt1(card)       ((card)->runtime_addr + DMA_CH_1_TRANS_BYTE_CNT)
#define DMADescrPtr1(card)      ((card)->runtime_addr + DMA_CH_1_DESC_PTR)
#define DMACmdStatus(card)      ((card)->runtime_addr + DMA_CMD_STATUS)
#define DMAArbitr(card)         ((card)->runtime_addr + DMA_MODE_ARB_REG)
#define DMAThreshold(card)      ((card)->runtime_addr + DMA_THRESHOLD_REG)

/* Mapping FIFO registers */
#define OutPostQIntStat(card)   ((card)->runtime_addr + OUT_POST_Q_INT_STATUS)
#define OutPostQIntMask(card)   ((card)->runtime_addr + OUT_POST_Q_INT_MASK)
#define MsgUnitCfg(card)        ((card)->runtime_addr + MSG_UNIT_CONFIG)
#define QBaseAddr(card)         ((card)->runtime_addr + Q_BASE_ADDR)
#define InFreeHeadPtr(card)     ((card)->runtime_addr + IN_FREE_HEAD_PTR)
#define InFreeTailPtr(card)     ((card)->runtime_addr + IN_FREE_TAIL_PTR)
#define InPostHeadPtr(card)     ((card)->runtime_addr + IN_POST_HEAD_PTR)
#define InPostTailPtr(card)     ((card)->runtime_addr + IN_POST_TAIL_PTR)
#define OutFreeHeadPtr(card)    ((card)->runtime_addr + OUT_FREE_HEAD_PTR)
#define OutFreeTailPtr(card)    ((card)->runtime_addr + OUT_FREE_TAIL_PTR)
#define OutPostHeadPtr(card)    ((card)->runtime_addr + OUT_POST_HEAD_PTR)
#define OutPostTailPtr(card)    ((card)->runtime_addr + OUT_POST_TAIL_PTR)
#define QStatusCtrl(card)       ((card)->runtime_addr + Q_STATUS_CTRL_REG)

/* Setup all the local registers */
#define BoardCtrlReg(card)      ((card)->local_addr + BOARD_CTRL_REG)
#define IntCtrlReg(card)		((card)->local_addr + INTERRUPT_CONTROL_REG)
#define InputBufferReg(card)    ((card)->local_addr + INPUT_DATA_BUFF_REG)
#define InputCtrlReg(card)      ((card)->local_addr + INPUT_BUFF_CTRL_REG)
#define RateAGenReg(card)		((card)->local_addr + RATE_A_GENERATOR_REG)
#define RateBGenReg(card)		((card)->local_addr + RATE_B_GENERATOR_REG)
#define BufferSizeReg(card)     ((card)->local_addr + BUFFER_SIZE_REG)
#define ScanSynchCtrlReg(card)  ((card)->local_addr + SCAN_SYNCH_CTRL_REG)

/*****************************************************************************/
/******                                                                 ******/
/******                           LOCAL ENUM                            ******/
/******                                                                 ******/
/*****************************************************************************/
enum 
{
    RX,
    TX
};
/*****************************************************************************/
/******                                                                 ******/
/******                           LOCAL TYPE                            ******/
/******                                                                 ******/
/*****************************************************************************/
typedef struct _PCI_BAR_INFO
{
	UINT32     *pVa;                        // BAR Kernel Virtual Address
	LARGE_INTEGER      Physical;                   // BAR Physical Address
	unsigned long      Size;                       // BAR size
	BOOL  IsIoMapped;					 // Memory or I/O mapped?
} PCI_BAR_INFO,*PPCI_BAR_INFO;

typedef struct _DMA_DATA_STRUCT 
{
    int dmaState;
    unsigned long *intermediateBuffer;        // DMA buffer
    unsigned long intermediatePhysicalAddr;   // physical address of above
    unsigned long userOffset;                 // current offset into user buffer.
    unsigned long userRemaining;              // space remaining in the user buffer.
    char *userBuffer;               // pointer to the current user buffer.
    unsigned long intermediateSamples;        // unread samples in the DMA buffer
    unsigned long intermediateStart;          // first unread sample in the DMA buffer
    unsigned long dmasamples;                 // unread samples in the DMA buffer
    unsigned long dmastart;                   // first unread sample in the DMA buffer
}DMA_DATA_STRUCT,*PDMA_DATA_STRUCT;

/* card descriptor structure */
typedef struct _DEVICE_BOARD 
{
	int busy;						/* device opened */
	ULONG						BusNumber;
	PCI_SLOT_NUMBER SlotNumber;
	PCI_BAR_INFO		PciBar[PCI_TYPE0_ADDRESSES];    // PCI BARs information
	ULONG						Irq; // IRQ number
	int							IntPriority;
	BOOL						UsingMSI;
	BOOL						bInitPending;
	BOOL						bAutocalPending;
	BOOL						bDmaInProgress;
	BOOL						bChReadyPending;
	BOOL						bLoHiPending;
	BOOL						bHiLoPending;
  HANDLE					hInterrupt;
	HANDLE					hEventInitComplete;
	HANDLE					hEventAutocalComplete;
	HANDLE					hEventChannelReady;
	HANDLE					hEventTriggerBurstInitialized;
	HANDLE					hEventTriggerBurstCompleted;
	HANDLE					hEventBufferThresLowToHigh;
	HANDLE					hEventBufferThresHighToLow;
	HANDLE					hEventBufferAlmostEmpty;
	HANDLE					hEventBufferAlmostFull;
	HANDLE					hEventDmaPending;
	unsigned long *runtime_addr;				/* remapped runtime register address space */
	unsigned long *local_addr;				/* remapped local register address space */
	int timeout_seconds;			/* time to wait before timeout (in seconds) */
	int fillBuffer;					/* Flag to wait for datatime to wait before timeout (in seconds) */
	unsigned long ulBufferThreshold;
	int detectOverflow;
  DMA_DATA_STRUCT dma_data[2];
  unsigned long dmasamples;                 // unread samples in the DMA buffer
  unsigned long dmastart;                   // first unread sample in the DMA buffer
  unsigned long hwBufferSize;
  unsigned long overflow_threshold;
}DEVICE_BOARD,*PDEVICE_BOARD;
/*****************************************************************************/
/******                                                                 ******/
/******                          STATIC DATA                            ******/
/******                                                                 ******/
/*****************************************************************************/
 
/*****************************************************************************/
/******                                                                 ******/
/******                     EXTERN OR GLOBAL DATA                       ******/
/******                                                                 ******/
/*****************************************************************************/
extern DEVICE_BOARD GSC16AI32_device[MAX_BOARDS];

/*****************************************************************************/
/******                                                                 ******/
/******                       FUNCTIONS PROTOTYPES                      ******/
/******                                                                 ******/
/*****************************************************************************/

//==============================================================================
//	Find Device
//==============================================================================
BOOL GSC16AI32_FindDevice(int	nDeviceNumber,
													int *pnBus,
													PCI_SLOT_NUMBER *pSlotNumber,	// output: pointer to slot number
													PPCI_COMMON_CONFIG  pPciData	// output: PCI card information
													);

void GSC16AI32_writePlx(PDEVICE_BOARD device,  unsigned long address, unsigned long value);
unsigned long GSC16AI32_readPlx(PDEVICE_BOARD device, unsigned long address);
void GSC16AI32_andPlx(PDEVICE_BOARD device, unsigned long address, unsigned long value);
void GSC16AI32_orPlx(PDEVICE_BOARD device, unsigned long address, unsigned long value);
void GSC16AI32_DisableIrqPlx(PDEVICE_BOARD device);
void GSC16AI32_EnableIrqPlx(PDEVICE_BOARD device);
void GSC16AI32_EnableIrqPlxDMA0(PDEVICE_BOARD device);

void GSC16AI32_writeLocal(PDEVICE_BOARD device, unsigned long address, unsigned long value);
unsigned GSC16AI32_readLocal(PDEVICE_BOARD device,unsigned long address);
void GSC16AI32_orLocal(PDEVICE_BOARD device,unsigned long address,unsigned long value );
void GSC16AI32_andLocal(PDEVICE_BOARD device,unsigned long address, unsigned long value );
void GSC16AI32_DisableIrqLocalAll(PDEVICE_BOARD device);
void GSC16AI32_DisableIrqLocal(PDEVICE_BOARD device, int group, unsigned long irq);
void GSC16AI32_EnableIrqLocal(PDEVICE_BOARD device, int group, unsigned long irq);
DWORD GSC16AI32_VirtualToPhysical(PVOID pvFrom);
BYTE* GSC16AI32_AllocateMemory(BOOL fContiguous, DWORD dwSizeInBytes); 
BYTE* GSC16AI32_MakeMemoryContiguous(BYTE *pbyBlock);
void GSC16AI32_FreeMemory(BOOL fContiguous, BYTE *pbyBlock);

int GSC16AI32_deviceInit(int nDeviceNumber);
void GSC16AI32_deviceCleanup(nDeviceNumber);
int GSC16AI32_deviceOpen(nDeviceNumber);
int GSC16AI32_deviceClose(nDeviceNumber);
size_t GSC16AI32_deviceRead(PDEVICE_BOARD device, char *buf, size_t size);

INTERRUPT_DISPOSITION RTFCNDCL GSC16AI32_DevicePending(PDEVICE_BOARD pDeviceInfo);
BOOLEAN RTFCNDCL GSC16AI32_DeviceIst(PDEVICE_BOARD pDeviceInfo);

#endif
