/*+[$HB0]===================================================================+
|                                                                         
|[$HI]--------------------------------------------------------------------
|                                                                         
| IDENTIFICATION     :                                                    
| --------------                                                          
| Project Name       : 
| File Name          : GSC16AI32.c                                         
| Target             : PC computer system (INTEL CPU)- Windows XP - RTX
| Development System : Visual 7.0                        
| Creation Date      :                                          
|                                                                         
|[$HD]--------------------------------------------------------------------
|                                                                         
| FILE DESCRIPTION   :                                                    
| ----------------                                                        
|                                                                         
|                                                                         
|[$HG]--------------------------------------------------------------------
|                                                                         
| GLOBAL DATA        :                                                    
| -----------                                                             
|                                                                         
|[$HV]--------------------------------------------------------------------
|                                                                         
| VERSION MANAGEMENT :                                                    
| ------------------                                                      
|   Author   |  Date    |     Comments                                    
| ----------------------------------------------------------------------  
|	           |          | [v1.0] Creation.                                
|            |          |                                                 
+[$HE0]===================================================================+*/
#include <windows.h>
#include <stdio.h>
#include <rtapi.h>
#include "plx_regs.h"
#include "internals.h"
#include "GSC16Ai32.h"

//==============================================================================
//
//	Initializes the board, i.e. reset
//
//==============================================================================
int GSC16AI32_Init(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	DWORD dwWaitRet;
	int nStatus=0;
//	unsigned int regval;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	nStatus=GSC16AI32_deviceInit(nDeviceNumber);
	if(nStatus != 0)
		return nStatus;
	nStatus=GSC16AI32_deviceOpen(nDeviceNumber);
	if(nStatus != 0)
		return nStatus;

	pDevice->bInitPending = TRUE;
	/*
	*	clear out any pending old interrupts.
	*/
	GSC16AI32_writeLocal(pDevice, INT_CTRL_REG,0);
	GSC16AI32_andPlx(pDevice,INT_CTRL_STATUS,~PCI_INT_ENABLE);

	/* enable PLX interrupt. */
	//GSC16AI32_writePlx(pDevice,INT_CTRL_STATUS,	IRQ_PCI_ENABLE | IRQ_LOCAL_PCI_ENABLE | IRQ_LOCAL_ENABLE);
	GSC16AI32_writePlx(pDevice,INT_CTRL_STATUS,	PCI_INT_ENABLE| IRQ_LOCAL_ENABLE);

	/* enable local init complete interrupt. */
	GSC16AI32_writeLocal(pDevice, INT_CTRL_REG,ICR_GROUP_0_IDLE|ICR_GROUP_0_REQUEST);


	GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,BCR_INITIALIZE);
/*	regval=GSC16AI32_readLocal(pDevice, BOARD_CTRL_REG);
	while((regval & 0x8000) != 0)
	{
		printf("BCR =%x\n",regval);
		RtSleep(1);
		regval=GSC16AI32_readLocal(pDevice, BOARD_CTRL_REG);
	}
	regval = GSC16AI32_readPlx(pDevice,INT_CTRL_STATUS);
	printf("plx INT_CTRL_STATUS =%x\n",regval);
	*/
	dwWaitRet=RtWaitForSingleObject(pDevice->hEventInitComplete,pDevice->timeout_seconds*1000);
	switch(dwWaitRet)
	{
	case WAIT_FAILED:
		pDevice->bInitPending = FALSE;
		nStatus=GSC16AI32_ERR_FAILED;
		break;
	case WAIT_ABANDONED:
		pDevice->bInitPending = FALSE;
		nStatus=GSC16AI32_ERR_ABANDONED;
		break;
	case WAIT_OBJECT_0:
		break;
	case WAIT_TIMEOUT:
		pDevice->bInitPending = FALSE;
		nStatus=GSC16AI32_ERR_TIMEOUT;
		break;
	}
	return nStatus;
}


//==============================================================================
//
//
//==============================================================================
int GSC16AI32_Close(int	nDeviceNumber)
{
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;

	nStatus = GSC16AI32_deviceClose(nDeviceNumber);
	GSC16AI32_deviceCleanup(nDeviceNumber);

	return nStatus;
}
//==============================================================================
//
//	Initiates Auto Calibration 
//
//==============================================================================
int GSC16AI32_Autocal(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	DWORD dwWaitRet;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	/* enable local interrupt. */
	regval = GSC16AI32_readLocal(pDevice,INT_CTRL_REG);
	regval &= ~ICR_GROUP_0_MASK;
	regval |=	ICR_GROUP_0_AUTOCAL_COMPLETE;
	pDevice->bAutocalPending = TRUE;
	GSC16AI32_writeLocal(pDevice, INT_CTRL_REG,regval);

	/* start autocal */
	regval = GSC16AI32_readLocal(pDevice,BOARD_CTRL_REG);
	//	  regval &= ~(ICR_0_MASK);
	regval |= BCR_AUTOCAL;
	pDevice->bAutocalPending = TRUE;
	/* enable PLX interrupt. */
	GSC16AI32_writePlx(pDevice,INT_CTRL_STATUS,PCI_INT_ENABLE  | IRQ_LOCAL_ENABLE);
	GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,regval);
	/*regval=GSC16AI32_readLocal(pDevice, BOARD_CTRL_REG);
	while((regval & 0x2000) != 0)
	{
		printf("BCR =%x\n",regval);
		RtSleep(1);
		regval=GSC16AI32_readLocal(pDevice, BOARD_CTRL_REG);
	}
	regval = GSC16AI32_readPlx(pDevice,INT_CTRL_STATUS);
	printf("plx INT_CTRL_STATUS =%x\n",regval);
	pDevice->bAutocalPending = FALSE;
*/	
	dwWaitRet=RtWaitForSingleObject(pDevice->hEventAutocalComplete,pDevice->timeout_seconds*1000);
	switch(dwWaitRet)
	{
	case WAIT_FAILED:
		pDevice->bAutocalPending = FALSE;
		nStatus=GSC16AI32_ERR_FAILED;
		break;
	case WAIT_ABANDONED:
		pDevice->bAutocalPending = FALSE;
		nStatus=GSC16AI32_ERR_ABANDONED;
		break;
	case WAIT_OBJECT_0:
		break;
	case WAIT_TIMEOUT:
		pDevice->bAutocalPending = FALSE;
		nStatus=GSC16AI32_ERR_AUTOCAL_FAILED;
		break;
	}
	return nStatus;
}


//==============================================================================
//	Read Local Register Data from the Device (32Bit)
//
//  See Users Manual for register bit definitions
//
//  Returns register value 
//==============================================================================
int GSC16AI32_ReadLocal32(int	nDeviceNumber,unsigned long usRegister,unsigned long * pulValue)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	*pulValue = GSC16AI32_readLocal(pDevice,usRegister);
	return nStatus;

}


//==============================================================================
//	Write Local Register Data to the Device (32Bit)
//
//  See Users Manual for register bit definitions
//
//==============================================================================
int GSC16AI32_WriteLocal32(int	nDeviceNumber,unsigned long usRegister,unsigned long ulValue)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_writeLocal(pDevice, usRegister, ulValue);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetDmaMode(int	nDeviceNumber,unsigned long ulValue)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if((ulValue < DMA_DISABLE) || (ulValue > DMA_DEMAND_MODE))
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		pDevice->dma_data[RX].dmaState = ulValue;
	}

	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputRangeConfig(int	nDeviceNumber,unsigned long ulRangeConfig)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulRangeConfig < INPUT_RANGE_2_5) || (ulRangeConfig > INPUT_RANGE_10)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{

		regval = GSC16AI32_readLocal(pDevice,BOARD_CTRL_REG);
		regval &= (~(BCR_RANGE_MASK));
		regval |= (ulRangeConfig<<BCR_RANGE_SHIFT);
		GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputModeConfig(int	nDeviceNumber,unsigned long ulModeConfig)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulModeConfig < INPUT_DIFFERENTIAL) || (ulModeConfig > INPUT_VREF_TEST)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{

		regval = GSC16AI32_readLocal(pDevice,BOARD_CTRL_REG);
		regval &= (~(INPUT_MODE_MASK));
		regval |= (ulModeConfig<<INPUT_MODE_SHIFT);
		// acquisition en complement a 2
		//regval &= ~0x40;
		GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,regval);
		//GSC16AI32_readLocal(pDevice,SCAN_SYNCH_CTRL_REG);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SSModeConfig(int	nDeviceNumber,unsigned long ulModeConfig)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulModeConfig < SS_SINGLE_ENDED) || (ulModeConfig > SS_FULL_DIFFERENTIAL)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{

		regval = GSC16AI32_readLocal(pDevice,BOARD_CTRL_REG);
		regval &= (~(SS_MODE_MASK));
		regval |= (ulModeConfig<<SS_MODE_SHIFT);
		GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputFormatConfig(int	nDeviceNumber,unsigned long ulFormatConfig)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulFormatConfig < FORMAT_TWO_COMPLEMENT) || (ulFormatConfig > FORMAT_OFFSET_BINARY)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{

		regval = GSC16AI32_readLocal(pDevice,BOARD_CTRL_REG);
		regval &= (~(BCR_FORMAT_MASK));
		regval |= (ulFormatConfig<<BCR_OFFSET_BINARY_SHIFT);
		GSC16AI32_writeLocal(pDevice, BOARD_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_StartSingleScan(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice, BOARD_CTRL_REG,BCR_INPUT_SYNCH_START);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanSize(int	nDeviceNumber,unsigned long ulScanSize)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulScanSize < SCAN_SINGLE_CHAN) || (ulScanSize > SCAN_32_CHAN)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{

		regval = GSC16AI32_readLocal(pDevice,SCAN_SYNCH_CTRL_REG);
		regval &= (~(SSCR_CHAN_SIZE_MASK));
		regval |= (ulScanSize);
		GSC16AI32_writeLocal(pDevice, SCAN_SYNCH_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanClock(int	nDeviceNumber,unsigned long ulScanClock)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulScanClock < EXTERNAL_SYNCH) || (ulScanClock > BCR_INPUT_SYNCH)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,SCAN_SYNCH_CTRL_REG);

		regval &= (~(SSCR_SCAN_CLOCK_MASK));
		regval |= (ulScanClock<<SSCR_CLOCK_SHIFT);
		GSC16AI32_writeLocal(pDevice, SCAN_SYNCH_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ScanClockEnable(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,SCAN_SYNCH_CTRL_REG,0x20);

	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ScanClockDisable(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,SCAN_SYNCH_CTRL_REG,~0x20);

	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisablePciInterrupt(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andPlx(pDevice,INT_CTRL_STATUS,~PCI_INT_ENABLE);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetBclockSource(int	nDeviceNumber,unsigned long ulBclockSource)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulBclockSource < MASTER_CLOCK) || (ulBclockSource > RATE_A_OUTPUT)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,SCAN_SYNCH_CTRL_REG);

		regval &= (~(SSCR_RATE_B_CLOCK_SOURCE_MASK));
		regval |= (ulBclockSource<<SSCR_RATE_B_CLOCK_SOURCE_SHIFT);
		GSC16AI32_writeLocal(pDevice, SCAN_SYNCH_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SingleChanSelect(int	nDeviceNumber,unsigned long ulChannel)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulChannel < 0) || (ulChannel > 32)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,SCAN_SYNCH_CTRL_REG);

		regval &= (~(SSCR_CHAN_SIZE_MASK));
		regval &= (~(SSCR_SINGLE_CHANNEL_MASK));
		regval |= (ulChannel<<SSCR_SINGLE_CHANNEL_SHIFT);
		GSC16AI32_writeLocal(pDevice, SCAN_SYNCH_CTRL_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanRateA(int	nDeviceNumber,unsigned long ulScanRateA)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulScanRateA < 0) || (ulScanRateA > 0xFFFF)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,RATE_A_GENERATOR_REG);

		regval &= (~(RGR_NRATE_MASK));
		regval |= (ulScanRateA);
		GSC16AI32_writeLocal(pDevice, RATE_A_GENERATOR_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanRateB(int	nDeviceNumber,unsigned long ulScanRateB)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulScanRateB < 0) || (ulScanRateB > 0xFFFF)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,RATE_B_GENERATOR_REG);

		regval &= (~(RGR_NRATE_MASK));
		regval |= (ulScanRateB);
		GSC16AI32_writeLocal(pDevice, RATE_B_GENERATOR_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenDisableA(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,RATE_A_GENERATOR_REG,RGR_GEN_DISABLE);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenDisableB(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,RATE_B_GENERATOR_REG,RGR_GEN_DISABLE);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenEnableA(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,RATE_A_GENERATOR_REG,~(RGR_GEN_DISABLE));
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenEnableB(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,RATE_B_GENERATOR_REG,~(RGR_GEN_DISABLE));
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetBufferThres(int	nDeviceNumber,unsigned long ulBufferThreshold)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulBufferThreshold < 0) || (ulBufferThreshold > 0x3FFFF)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,INPUT_BUFF_CTRL_REG);
		regval &= (~(0x3FFFF));
		regval |= (ulBufferThreshold);
		pDevice->ulBufferThreshold = regval;
		GSC16AI32_writeLocal(pDevice, INPUT_BUFF_CTRL_REG,regval);
		pDevice->bLoHiPending = TRUE;
		GSC16AI32_EnableIrqLocal(pDevice, GROUP_1, ICR_GROUP_1_LO_HI);
		GSC16AI32_EnableIrqPlx(pDevice);


	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ClearBuffer(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,INPUT_BUFF_CTRL_REG,IDC_INPUT_CLEAR_BUFFER);
	return nStatus;
}
/*======================================================================
 * NAME : GSC16AI32_extractDeviceNumberFromRessourceName()
 *
 * DESCRIPTION: find the device number from the ressource name
 *		
 * ARGUMENTS:
 *		szRessourceName	:	name of the ressource
 *
 * RETURN VALUE:
 *		Device Number or -1 if not found
 *
 *=====================================================================*/
int GSC16AI32_extractDeviceNumberFromRessourceName(char *szRessourceName)
{
	int iDeviceNumber = -1;
	char cDeviceNumber;
	int i = 1;

	while ( (i <= (int) strlen(szRessourceName)) && (iDeviceNumber == -1) )
	{
		cDeviceNumber = szRessourceName[strlen(szRessourceName)-i];
		if (cDeviceNumber >= '0' || cDeviceNumber <= '9')
			iDeviceNumber = cDeviceNumber - '0' ;
		i++;
	}

	return iDeviceNumber ;

}
/*======================================================================
 * NAME : GSC16AI32_revisionQuery()
 *
 * DESCRIPTION: 
 *		
 * ARGUMENTS:
 *
 * RETURN VALUE:
 *
 *=====================================================================*/
int GSC16AI32_revisionQuery(int	nDeviceNumber, char szDriverRevision[255], char szFirmwareRevision[255] )
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	int nNumChan = 0;
	int nMasterClock = 0;
	BOOL bTimeStamp;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	regval = GSC16AI32_readLocal(pDevice,FIRMWARE_REV_REG);
	if((regval&0x30000) == 0)
	{
		nNumChan = 32;
	}
	else if((regval&0x30000) == 0x10000)
	{
		nNumChan = 16;
	}
	if((regval&0xC0000) == 0)
	{
		nMasterClock = 50;
	}
	if((regval&0x300000) == 0x100000)
	{
		bTimeStamp = TRUE;
	}
	else
	{
		bTimeStamp = FALSE;
	}
  
	sprintf(szDriverRevision,"%s version %S",GSC_NAME,DRIVER_VERSION);
	sprintf(szFirmwareRevision,"FW_REV = %08lX : %02d Channels : MASTER CLOCK %dMHz : TIMESTAMP =%01d  ",regval,nNumChan,nMasterClock,bTimeStamp);
	return nStatus;

}
/*======================================================================
 * NAME : GSC16AI32_ReadChannel()
 *
 * DESCRIPTION: 
 *		
 * ARGUMENTS:
 *
 * RETURN VALUE:
 *
 *=====================================================================*/
int GSC16AI32_ReadChannel(int	nDeviceNumber,int nChannelNumber, unsigned short * pusChannelValue)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	return nStatus;
}
/*======================================================================
 * NAME : GSC16AI32_ReadBuffer()
 *
 * DESCRIPTION: 
 *		
 * ARGUMENTS:
 *
 * RETURN VALUE:
 *
 *=====================================================================*/
size_t GSC16AI32_ReadBuffer(int	nDeviceNumber, char *buf, size_t size)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	return GSC16AI32_deviceRead(pDevice, buf, size);
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTag(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,BOARD_CTRL_REG,BCR_ENABLE_TIME_TAG);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTag(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,BOARD_CTRL_REG,~BCR_ENABLE_TIME_TAG);
	return nStatus;
}

//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagConfScanClock(int	nDeviceNumber,unsigned long ulConfScanClock)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if ((ulConfScanClock < TTC_INTERNAL_RATE_A) || (ulConfScanClock > TTC_EXTERNAL_REF_CLK)) 
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	else
	{
		regval = GSC16AI32_readLocal(pDevice,TIME_TAG_CONF_REG);

		regval &= (~(TTC_SCAN_CLOCK_MASK));
		regval |= (ulConfScanClock<<TTC_SCAN_CLOCK_SHIFT);
		GSC16AI32_writeLocal(pDevice, TIME_TAG_CONF_REG,regval);
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenTimeTagEnable(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,TTC_ENABLE_SAMPLING);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenTimeTagDisable(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~TTC_ENABLE_SAMPLING);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagConf(int	nDeviceNumber,unsigned long ulConfClock)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if(ulConfClock == TTC_INTERNAL)
	{
		GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x100);

	}
	else if(ulConfClock == TTC_EXTERNAL)
	{
		GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x100);
	}
	else
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}

	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ResetTimeTag(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x200);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ResetTimeTagConf(int	nDeviceNumber,unsigned long ulResetTimeTagConf)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	if(ulResetTimeTagConf == TTC_INTERNAL_RESET)
	{
		GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x400);

	}
	else if(ulResetTimeTagConf == TTC_EXTERNAL_RESET)
	{
		GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x400);
	}
	else
	{
		nStatus = GSC16AI32_ERR_INVALID_PARAMETER;
	}
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTagHeader(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x800);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTagHeader(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x800);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetChannelThreshold(int	nDeviceNumber,int nChannelNumber, unsigned short usValue)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	regval = GSC16AI32_readLocal(pDevice,CH0_THRES_REF_REG+nChannelNumber);
	regval = regval & 0xFFFF;
	regval = regval | (usValue << 16);
	GSC16AI32_writeLocal(pDevice, CH0_THRES_REF_REG+nChannelNumber,regval);
	return nStatus;

}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetChannelReference(int	nDeviceNumber,int nChannelNumber, unsigned short usValue)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	regval = GSC16AI32_readLocal(pDevice,CH0_THRES_REF_REG+nChannelNumber);
	regval = regval & 0xFFFF0000;
	regval = regval | usValue;
	GSC16AI32_writeLocal(pDevice, CH0_THRES_REF_REG+nChannelNumber,regval);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTriggerMode(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	unsigned long regval;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x10);
	regval = GSC16AI32_readLocal(pDevice,INT_CTRL_REG);
	regval &= ~ICR_GROUP_0_MASK;
	regval |=	ICR_GROUP_0_TRIGGER_BURST_COMPLETED;
	//regval |=	0x8;
	GSC16AI32_writeLocal(pDevice, INT_CTRL_REG,regval);
	GSC16AI32_writePlx(pDevice,INT_CTRL_STATUS,	PCI_INT_ENABLE| IRQ_LOCAL_ENABLE);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTriggerMode(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x10);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableGlobalTriggering(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x40);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableGlobalTriggering(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x40);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableUpdateRefVal(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,TIME_TAG_CONF_REG,~0x20);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableUpdateRefVal(int	nDeviceNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,TIME_TAG_CONF_REG,0x20);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTagChannel(int	nDeviceNumber,int nChannelNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_orLocal(pDevice,ACTIVE_CH_MASK_REG,1<<nChannelNumber);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTagChannel(int	nDeviceNumber,int nChannelNumber)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_andLocal(pDevice,ACTIVE_CH_MASK_REG,~(1<<nChannelNumber));
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagActiveChannelMask(int	nDeviceNumber,unsigned long ulActiveChannelMask)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;
	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_writeLocal(pDevice,ACTIVE_CH_MASK_REG,ulActiveChannelMask);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagBurstSize(int	nDeviceNumber,unsigned long ulBurstSize)
{
	PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	GSC16AI32_writeLocal(pDevice, NB_SAMPLES_AFTER_TRIG_REG,ulBurstSize);
	return nStatus;
}
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GetEventHandle(int	nDeviceNumber,int nEventType,HANDLE *phEvent)
{
		PDEVICE_BOARD pDevice;
	int nStatus=0;

	if((nDeviceNumber < 0) || (nDeviceNumber > MAX_BOARDS))
		return GSC16AI32_ERR_INVALID_PARAMETER;
	pDevice=&GSC16AI32_device[nDeviceNumber];
	switch (nEventType)
	{
	case TRIGGER_BURST_COMPLETED:
		*phEvent = pDevice->hEventTriggerBurstCompleted;
		break;
	case THRES_LO_HI:
		*phEvent = pDevice->hEventBufferThresLowToHigh;
		break;
	default:
		return GSC16AI32_ERR_INVALID_PARAMETER;
	}
	return nStatus;

}
