/***
*** hardware.c  
***
***  General description of this file:
***     Device driver source code for General Standards 16AI64
***     family of analog input boards. This file is part of the Linux
***     driver source distribution for this board.
***     
***
***  Author:
***     
***
***  Support:
***     Primary support for this driver is provided by GSC. 
***
***/
//////////////////////////////////////////////////////////////////////////
// set the following flag to trace interrupt debug messages.
#include <windows.h>
#include <stdio.h>
#include <rtapi.h>

#include "plx_regs.h"
#include "internals.h"
#include "GSC16Ai32.h"


static int groupMasks[] =
{
    ICR_GROUP_0_MASK<<ICR_GROUP_0_SHIFT,
		ICR_GROUP_1_MASK<<ICR_GROUP_1_SHIFT,
};

static int groupShifts[] =
{
    ICR_GROUP_0_SHIFT,
    ICR_GROUP_1_SHIFT,
};
/************************************************************************/
/* writeplx                                                           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_writePlx(PDEVICE_BOARD device, unsigned long address,unsigned long value)
{
	*(device->runtime_addr+address)=value;
};

/************************************************************************/
/* readplx                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
unsigned long GSC16AI32_readPlx(PDEVICE_BOARD device, unsigned long address)
{
   return 	*(device->runtime_addr+address);

};
/************************************************************************/
/* andplx                                                           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_andPlx(PDEVICE_BOARD device, unsigned long address,unsigned long value)
{
  unsigned long regval;
	regval = *(device->runtime_addr+address);
	regval &= value;

	*(device->runtime_addr+address)= regval;
};
/************************************************************************/
/* orplx                                                           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_orPlx(PDEVICE_BOARD device, unsigned long address,unsigned long value)
{
  unsigned long regval;
	regval = *(device->runtime_addr+address);
	regval |= value;

	*(device->runtime_addr+address)= regval;
};
/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_DisableIrqPlx(PDEVICE_BOARD device)
{
	*(device->runtime_addr + INT_CTRL_STATUS) = 1;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_EnableIrqPlx(PDEVICE_BOARD device)
{
 GSC16AI32_orPlx(device, INT_CTRL_STATUS, (IRQ_PCI_ENABLE | IRQ_LOCAL_PCI_ENABLE));
}


/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_EnableIrqPlxDMA0(PDEVICE_BOARD device)
{
 GSC16AI32_orPlx(device, INT_CTRL_STATUS, IRQ_PCI_ENABLE | IRQ_DMA_0_ENABLE);
}



/************************************************************************/
/* writeLocal                                                           */
/*                                                                      */
/* Write to the board Local registers, whether the Local registers      */
/* are in I/O space or memory space.                                    */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_writeLocal(PDEVICE_BOARD device, unsigned long address,unsigned long value )
{
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
				RtWritePortUlong(device->local_addr+address,value);
    }
    else
    {
				*(device->local_addr+address)=value;
    }
};

/************************************************************************/
/* readLocal                                                            */
/*                                                                      */
/* Read from the board local registers, whether the local registers     */
/* are in I/O space or memory space.                                    */
/*                                                                      */
/*                                                                      */
/************************************************************************/
unsigned GSC16AI32_readLocal(PDEVICE_BOARD device, unsigned long address)
{
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
				return RtReadPortUlong(device->local_addr+address);
    }
    else
    {
        return *(device->local_addr+address);
    }
};
/************************************************************************/
/* andlocal                                                           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_andLocal(PDEVICE_BOARD device, unsigned long address,unsigned long value)
{
  unsigned long regval;
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
			regval = RtReadPortUlong(device->local_addr+address);
			regval &= value;
			RtWritePortUlong(device->local_addr+address,regval);
    }
    else
    {
			regval = *(device->local_addr+address);
			regval &= value;
			*(device->local_addr+address)=regval;
    }
};
/************************************************************************/
/* orlocal                                                           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_orLocal(PDEVICE_BOARD device, unsigned long address,unsigned long value)
{
  unsigned long regval;
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
			regval = RtReadPortUlong(device->local_addr+address);
			regval |= value;
			RtWritePortUlong(device->local_addr+address,regval);
    }
    else
    {
			regval = *(device->local_addr+address);
			regval |= value;
			*(device->local_addr+address)=regval;
    }
};
/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_DisableIrqLocalAll(PDEVICE_BOARD device)
{
    GSC16AI32_writeLocal(device,INT_CTRL_REG,0);
}
/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_EnableIrqLocal(PDEVICE_BOARD device, int group, unsigned long irq)
{
    unsigned long regval;

    regval = GSC16AI32_readLocal(device,INT_CTRL_REG);
    regval &= (~(groupMasks[group]));
    regval |= (irq << groupShifts[group]);
		if(group == GROUP_0)
		{
			regval &= 0xFFFFFFF8;
			regval |= (irq & 0x7);
			//regval |= 0x8;
		}
		else
		{
			regval &= 0xFFFFFF8F;
			regval |= (irq & 0x7)<< 4;
			//regval |= 0x80;
		}
    GSC16AI32_writeLocal(device,INT_CTRL_REG,regval);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void GSC16AI32_DisableIrqLocal(PDEVICE_BOARD device, int group, unsigned long irq)
{
    unsigned long regval;

    regval = GSC16AI32_readLocal(device,INT_CTRL_REG);
    regval &= (~(groupMasks[group]));
    GSC16AI32_writeLocal(device,INT_CTRL_REG,regval);
}

/************************************************************************/
/* FindDevice                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
BOOL GSC16AI32_FindDevice(		
													int	nBoardNumber,
													int *pnBus,
													PCI_SLOT_NUMBER *pSlotNumber,	// output: pointer to slot number
													PPCI_COMMON_CONFIG  pPciData	// output: PCI card information
													)
{
	BOOL					bStatus;
	unsigned long	ulBytesReturned;	
	int	nBusNumber;			// between 0 and PCI_MAX_BUS 	
	int nDeviceNumber;		// between 0 and PCI_MAX_DEVICES 
	int nFunctionNumber;		// between 0 and PCI_MAX_FUNCTION
	int nRang;
	int bytesWritten;
	BOOL bFlag = TRUE;           

	//*** Tests parameters

	//*** Initialisation
	bStatus = 0;
	ulBytesReturned = 0;
	nBusNumber = 0; 
	nDeviceNumber = 0;
	nFunctionNumber = 0;
	nRang = -1;
	// 
	// All PCI cards have been searched when we are out of PCI busses. (bFlag = FALSE) 
	//

	//*** Traitement
	for (nBusNumber=0; bFlag; nBusNumber++)
	{
		for (nDeviceNumber = 0 ; nDeviceNumber < PCI_MAX_DEVICES ; nDeviceNumber++)
		{
			pSlotNumber->u.bits.DeviceNumber = nDeviceNumber; 

			for (nFunctionNumber=0; nFunctionNumber<PCI_MAX_FUNCTION; nFunctionNumber++)
			{
				pSlotNumber->u.bits.FunctionNumber = nFunctionNumber; 

				bytesWritten = RtGetBusDataByOffset (
					PCIConfiguration,		// type of bus data to be retrieved
					nBusNumber,					// zero-based number of the bus 
					pSlotNumber->u.AsULONG,	// logical slot number
					pPciData,				// pointer to a buffer for configuration information 
					0,						// byte offset into buffer
					PCI_COMMON_HDR_LENGTH	// length of buffer
					); 

				if ( bytesWritten == 0 )
				{ 
					// out of PCI buses done
					bFlag = FALSE; 
					break; 
				} 

				if ( bytesWritten == VENDOR_ID_SIZE && pPciData->VendorID == PCI_INVALID_VENDORID )
				{ 
					// no device at this slot number, skip to next slot 
					break; 
				} 

				//
				//  If device is found return, otherwise continue 
				//  until all buses have been searched.
				//	Base Class and sub Class can also be used to find your
				//	specific device.
				// 
				if ( ( pPciData->VendorID == PCI_VENDOR_ID_PLX ) && ( pPciData->DeviceID == PCI_DEVICE_ID_PLX_9056 ) 
					&& (pPciData->u.type0.SubVendorID == GSC_SUBVENDOR) && (pPciData->u.type0.SubSystemID == GSE_SUBDEVICE16))
				{
					nRang++;
				}
				if(nRang == nBoardNumber)
				{
					*pnBus = nBusNumber;
					return TRUE;
				}
			}
		}
	}
	return FALSE;

}
/************************************************************************/
/* GSC16AI32_VirtualToPhysical                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
DWORD GSC16AI32_VirtualToPhysical(PVOID pvFrom)
{
	LARGE_INTEGER physaddrTo;

	physaddrTo = RtGetPhysicalAddress (pvFrom);
	return (physaddrTo.LowPart);
}

/************************************************************************/
/* GSC16AI32_AllocateMemory                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
BYTE* GSC16AI32_AllocateMemory(BOOL fContiguous, DWORD dwSizeInBytes) 
{
	BYTE* pvResult = NULL;
	LARGE_INTEGER physaddrHighest;
	if (fContiguous)
	{

		physaddrHighest.QuadPart = 0xFFFFFFFF;
		pvResult = ((BYTE*)RtAllocateContiguousMemory(dwSizeInBytes, physaddrHighest));
	}
	else
	{
		pvResult = ((BYTE*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (dwSizeInBytes + PAGE_SIZE)));
	}
	return (pvResult);
}

/************************************************************************/
/* GSC16AI32_MakeMemoryContiguous                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
BYTE* GSC16AI32_MakeMemoryContiguous(BYTE *pbyBlock)
{
	DWORD dwPageSize = PAGE_SIZE;
	DWORD dwOffset;
	DWORD dwTemp;

	dwTemp = GSC16AI32_VirtualToPhysical (pbyBlock);
	dwTemp &= (dwPageSize - 1);
	dwOffset = 0;
	if (dwTemp != 0)
	{
		dwOffset = (dwPageSize - dwTemp);
	}
	return (pbyBlock + dwOffset);
}

/************************************************************************/
/* GSC16AI32_FreeMemory                                                            */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void GSC16AI32_FreeMemory(BOOL fContiguous, BYTE *pbyBlock)
{
	if (fContiguous)
	{
		RtFreeContiguousMemory ((PVOID) pbyBlock); 
	}
	else
	{
		HeapFree (GetProcessHeap (), 0, ((PVOID) pbyBlock));
	}
}
