/***
*** init.c
***
***  General description of this file:
***     Device driver source code for General Standards 16AI64
***     family of analog input boards. This file is part of the Linux
***     driver source distribution for this board.
***
***
***  Author:
***
***  Support:
***
***/

//////////////////////////////////////////////////////////////////////////
// set the following flag to trace debug messages in this module.
#include <windows.h>
#include <stdio.h>
#include <rtapi.h>
#include "plx_regs.h"
#include "internals.h"
#include "GSC16Ai32.h"


const char *event_name[]=
{
	"EVENT_INIT_COMPLETE",
	"EVENT_AUTOCAL_COMPLETE",
	"EVENT_CHANNELS_READY",
	"EVENT_TRIGGER_BURST_INITIALIZED",
	"EVENT_TRIGGER_BURST_COMPLETED",
	"EVENT_BUFFER_THRES_LOW_TO_HIGH",
	"EVENT_BUFFER_THRES_HIGH_TO_LOW",
	"EVENT_BUFFER_ALMOST_EMPTY",
	"EVENT_BUFFER_ALMOST_FULL",
	"EVENT_DMA_PENDING",
	"EVENT_LAST_IRQ_EVENT",
};
DEVICE_BOARD GSC16AI32_device[MAX_BOARDS];

/************************************************************************/
/* module initialization: detect card(s) and set up descriptor(s)        */
/************************************************************************/
int GSC16AI32_deviceInit(int nDeviceNumber)
{
	DEVICE_BOARD *device;
	int i;
	BOOL bFound;
	int nBus;
	PCI_SLOT_NUMBER SlotNumber;
	PCI_COMMON_CONFIG PciData;
	LARGE_INTEGER memAddr;		// a base port address 
	LARGE_INTEGER tranMemAddr;	// translated base addresses returned by RtMapMemory
	ULONG AddressSpace = 0;     // indicates memory space
	SlotNumber.u.AsULONG = 0L;

	device= &GSC16AI32_device[nDeviceNumber];
	memset(device,0,sizeof(DEVICE_BOARD));
	bFound =GSC16AI32_FindDevice(nDeviceNumber,&nBus,&SlotNumber,&PciData);
	if(bFound == TRUE)
	{
		//pci_read_config_word(pdev, 0x2E, &reg);
		//RtPrintf(":     config reg=0x%x\n", reg);
		//RtPrintf(":     attaching board #%d\n", index + 1);
		device->BusNumber = nBus;
		device->SlotNumber.u.AsULONG = SlotNumber.u.AsULONG;

		for (i = 0; i < PCI_TYPE0_ADDRESSES; ++i)
		{
			memAddr.QuadPart = PciData.u.type0.BaseAddresses[i]; 
			// Verify the address is valid
			if (!RtTranslateBusAddress ( 
				PCIBus,			// bus interface type
				nBus,		// bus number (zero based)
				memAddr,		// bus-relative address
				&AddressSpace,	// specifies port number or a memory address
				&tranMemAddr	// pointer to the translated address
				) )
			{
				continue;
			}

			// Get PCI physical address
			device->PciBar[i].Physical.QuadPart = tranMemAddr.QuadPart;

			// Determine resource type
			if (AddressSpace & 0x1)
			{

				// Make sure flags are cleared properly
				device->PciBar[i].IsIoMapped         = TRUE;
			}
			else
			{

				// Make sure flags are cleared properly
				device->PciBar[i].IsIoMapped         = FALSE;
			}

			// Get the size
			device->PciBar[i].Size = 4096; /*
																		 Plx_pci_resource_len(
																		 device,
																		 i
																		 );*/
		}
	}
	else
	{
		return GSC16AI32_ERR_CARD_NOT_FOUND;
	}

	device->runtime_addr = RtMapMemory(device->PciBar[0].Physical, 4096,FALSE);
	if(device->runtime_addr == NULL)
	{
		return GSC16AI32_ERR_MAPPING_MEMORY;
	}
	if (device->PciBar[2].IsIoMapped)
	{
		RtPrintf(": I/O mapped local registers\n");
		device->local_addr = (unsigned long *)device->PciBar[2].Physical.LowPart;
		RtEnablePortIo((unsigned char *)device->local_addr,4096);
	}
	else
	{
		device->local_addr = RtMapMemory(device->PciBar[2].Physical, 4096,FALSE);
		if(device->local_addr == NULL)
		{
			RtUnmapMemory(device->runtime_addr);
			device->runtime_addr = NULL;
			return GSC16AI32_ERR_MAPPING_MEMORY;

		}
	}
	device->Irq = PciData.u.type0.InterruptLine;


	return 0;
}

/************************************************************************/
/* cleanup when unloading module                                        */
/************************************************************************/
void GSC16AI32_deviceCleanup(nDeviceNumber)
{
	DEVICE_BOARD *device;
	device= &GSC16AI32_device[nDeviceNumber];

	if (device->PciBar[2].IsIoMapped)
	{
		RtDisablePortIo((unsigned char *)device->local_addr,4096);
		device->local_addr = NULL;
	}
	else
	{
		RtUnmapMemory(device->local_addr);
		device->local_addr = NULL;
	}

	RtUnmapMemory(device->runtime_addr);
	device->runtime_addr = NULL;
}

/************************************************************************/
/* open device                                                          */
/************************************************************************/
int GSC16AI32_deviceOpen(nDeviceNumber)
{
	DEVICE_BOARD *device;
#ifdef ATTACH_LINE_BASED
	ATTACH_INTERRUPT_PARAMETERS AttachParams; // Attach interrupt parameters
#endif
	char szEventName[80];

	device = &GSC16AI32_device[nDeviceNumber];

	if (device->busy) 
	{
		return (GSC16AI32_ERR_CARD_ALLREADY_INITIALIZED);
	}
	sprintf(szEventName,"%s_%d",event_name[0],nDeviceNumber);
	device->hEventInitComplete=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventInitComplete == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[1],nDeviceNumber);
	device->hEventAutocalComplete=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventAutocalComplete == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[2],nDeviceNumber);
	device->hEventChannelReady=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventChannelReady == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[3],nDeviceNumber);
	device->hEventTriggerBurstInitialized=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventChannelReady == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[4],nDeviceNumber);
	device->hEventTriggerBurstCompleted=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventChannelReady == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[5],nDeviceNumber);
	device->hEventBufferThresLowToHigh=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventBufferThresLowToHigh == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[6],nDeviceNumber);
	device->hEventBufferThresHighToLow=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventBufferThresHighToLow == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[7],nDeviceNumber);
	device->hEventBufferAlmostEmpty=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventBufferAlmostEmpty == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[8],nDeviceNumber);
	device->hEventBufferAlmostFull=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventBufferAlmostFull == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
	sprintf(szEventName,"%s_%d",event_name[9],nDeviceNumber);
	device->hEventDmaPending=RtCreateEvent(NULL,FALSE,FALSE,szEventName);
	if(device->hEventDmaPending == NULL)
	{
		return (GSC16AI32_ERR_CREATE_EVENT);
	}
#ifdef ATTACH_LINE_BASED
	// Select line-based attach if device does not support MSI or input for line-based
	if(!RtQueryPciMsiCapability(device->BusNumber, device->SlotNumber))
	{
		AttachParams.AttachVersion = ATTACH_LINE_BASED;             // Line based version
		AttachParams.LineBased.pThreadAttributes = NULL;            // Security attributes (default)
		AttachParams.LineBased.StackSize = 0;                       // Use default value
		AttachParams.LineBased.pRoutine = GSC16AI32_DeviceIst;             // IST
		AttachParams.LineBased.Context = device;                 // Context passed to IST and ISR
		AttachParams.LineBased.Priority = 127;   // IST priority
		AttachParams.LineBased.InterfaceType = PCIBus;              // PCI bus
		AttachParams.LineBased.BusNumber = device->BusNumber;    // Bus number
		AttachParams.LineBased.SlotNumber = device->SlotNumber;  // Slot number
		AttachParams.LineBased.BusInterruptLevel = device->Irq;  // Interrupt level
		AttachParams.LineBased.BusInterruptVector = device->Irq; // Interrupt vector
		AttachParams.LineBased.Shared = TRUE;                      // Not shared
		AttachParams.LineBased.InterruptMode = LevelSensitive;      // Type of interrupt
		AttachParams.LineBased.MyInterrupt = GSC16AI32_DevicePending;                  // ISR
		AttachParams.LineBased.ProcessorEnableMask =
			1 << RtGetCurrentProcessorNumber();                     // Rtss Processor Mask
		device->UsingMSI = FALSE;
	}
	else
	{
		AttachParams.AttachVersion = ATTACH_MESSAGE_BASED;            // Message based version
		AttachParams.MessageBased.pThreadAttributes = NULL;           // Security attributes (default)
		AttachParams.MessageBased.StackSize = 0;                      // Use default value
		AttachParams.MessageBased.pRoutine = GSC16AI32_DeviceIst;            // IST
		AttachParams.MessageBased.Context = device;                // Context passed to IST and ISR
		AttachParams.MessageBased.Priority = device->IntPriority;  // IST priority
		AttachParams.MessageBased.BusNumber = device->BusNumber;   // Bus number
		AttachParams.MessageBased.SlotNumber = device->SlotNumber; // Slot number
		AttachParams.MessageBased.MyInterrupt = GSC16AI32_DevicePending;   // ISR
		AttachParams.MessageBased.ProcessorEnableMask =
			1 << RtGetCurrentProcessorNumber();                       // Rtss Processor Mask
		device->UsingMSI = TRUE;
	}

	// Attach interrupt
	device->hInterrupt = RtAttachInterrupt(&AttachParams);
	//msg("(%d): opening board\n", device->minor);
	if (device->hInterrupt == NULL) 
	{
		device->busy = 0;
		return (GSC16AI32_ERR_ATTACH_INTERRUPT);
	}
#else
	device->hInterrupt = RtAttachInterruptVectorEx(NULL,
                        0,                        
                        GSC16AI32_DeviceIst,
                        device,
                        127,
                        PCIBus,
                        device->BusNumber,
                        device->Irq,
                        device->Irq,
                        TRUE,
                        LevelSensitive,
                        GSC16AI32_DevicePending);
	if (device->hInterrupt == NULL) 
	{
		device->busy = 0;
		return (GSC16AI32_ERR_ATTACH_INTERRUPT);
	}
#endif
	if (!(device->dma_data[RX].intermediateBuffer = (unsigned long *)GSC16AI32_AllocateMemory(TRUE, DMA_SIZE))) 
	{
		if (device->hInterrupt != NULL)
		{
#ifdef ATTACH_LINE_BASED
			RtReleaseInterrupt(device->hInterrupt);
#else
			RtReleaseInterruptVector(device->hInterrupt);
#endif

		}
		device->busy = 0;
		return (GSC16AI32_ERR_ALLOC_DMA_BUFFER);
	}
	device->dma_data[RX].intermediatePhysicalAddr = GSC16AI32_VirtualToPhysical((void *)device->dma_data[RX].intermediateBuffer);

	device->timeout_seconds=20;
	device->busy = 1;
	device->dma_data[RX].dmaState = DMA_DISABLE;
	device->dma_data[RX].dmastart = FALSE;
	device->dma_data[RX].dmasamples = 0;
	device->dma_data[RX].intermediateSamples=0;
	device->overflow_threshold = INPUT_BUFFER_SIZE-0x100;
	device->detectOverflow=FALSE;


	device->hwBufferSize=HW_BUFFER_SIZE;


	// disable local interrupts
	GSC16AI32_DisableIrqLocal(device,0,0);
	return (0);
}

/************************************************************************/
/* close device                                                         */
/************************************************************************/
int GSC16AI32_deviceClose(nDeviceNumber)
{
	DEVICE_BOARD *device;
	device = &GSC16AI32_device[nDeviceNumber];

	//RtPrintf("(%d): closing board\n", device->minor);
	GSC16AI32_DisableIrqPlx(device);

	/* reset DMA engines */
	GSC16AI32_andPlx(device, DMA_CMD_STATUS, STOP_DMA_CMD_0_MASK);
	GSC16AI32_andPlx(device, DMA_CMD_STATUS, STOP_DMA_CMD_1_MASK);

	/* reset local IRQ flag */
	GSC16AI32_DisableIrqLocal(device,0,0);

	/* free resources */
	if(device->hInterrupt != NULL)
	{
#ifdef ATTACH_LINE_BASED
		RtReleaseInterrupt(device->hInterrupt);
#else
		RtReleaseInterruptVector(device->hInterrupt);
#endif
	}
	RtCloseHandle(device->hEventInitComplete);
	device->hEventInitComplete = NULL;
	RtCloseHandle(device->hEventAutocalComplete);
	device->hEventAutocalComplete = NULL;
	RtCloseHandle(device->hEventChannelReady);
	device->hEventChannelReady = NULL;
	RtCloseHandle(device->hEventTriggerBurstInitialized);
	device->hEventTriggerBurstInitialized = NULL;
	RtCloseHandle(device->hEventTriggerBurstCompleted);
	device->hEventTriggerBurstCompleted = NULL;
	RtCloseHandle(device->hEventBufferThresLowToHigh);
	device->hEventBufferThresLowToHigh = NULL;
	RtCloseHandle(device->hEventBufferThresHighToLow);
	device->hEventBufferThresHighToLow = NULL;
	RtCloseHandle(device->hEventBufferAlmostEmpty);
	device->hEventBufferAlmostEmpty = NULL;
	RtCloseHandle(device->hEventBufferAlmostFull);
	device->hEventBufferAlmostFull = NULL;
	RtCloseHandle(device->hEventDmaPending);
	device->hEventDmaPending = NULL;
	GSC16AI32_FreeMemory(TRUE, (BYTE *)device->dma_data[RX].intermediateBuffer);
	device->busy = 0;
	return (0);
}
