#include <windows.h>
#include <stdio.h>
#include <rtapi.h>
#include "plx_regs.h"
#include "internals.h"
#include "GSC16Ai32.h"

INTERRUPT_DISPOSITION RTFCNDCL GSC16AI32_DevicePending(PDEVICE_BOARD pDeviceInfo)
{
	BOOL fRecvEvent = FALSE;
	unsigned long plx_irq_reg;

	plx_irq_reg = GSC16AI32_readPlx(pDeviceInfo,INT_CTRL_STATUS);

    if (((plx_irq_reg & IRQ_PCI_ENABLE)==0) ||
        (((plx_irq_reg & IRQ_DMA_0_ACTIVE)==0)&&((plx_irq_reg & IRQ_LOCAL_ACTIVE))==0))
		{
			return PassToNextDevice;
		}
		else
		{
			return CallInterruptThread;
		}
}

/************************************************************************/
/* Interrupt handler                                                    */
/************************************************************************/
BOOLEAN RTFCNDCL  GSC16AI32_DeviceIst(PDEVICE_BOARD pDeviceInfo)
{
	PDEVICE_BOARD device = pDeviceInfo;
	unsigned long local_irq_reg;
	unsigned long plx_irq_reg;
	unsigned long dma_reg;
	int irq_type;

	//printk(KERN_INFO DEVICE_NAME " Interrupt()\n");
	/* determine if the interrupt is from this board. */

	local_irq_reg = GSC16AI32_readLocal(pDeviceInfo,INT_CTRL_REG);
	plx_irq_reg = GSC16AI32_readPlx(pDeviceInfo,INT_CTRL_STATUS);

	if ((plx_irq_reg & IRQ_PCI_ENABLE) == 0) 
	{ /* not ours */
		printf("Interrupt not ours... %.8lx\n",plx_irq_reg);
		return FALSE;
	}
	//printk(KERN_INFO DEVICE_NAME " Interrupt->ours! plx irq: %.8lX local irq: %.8lX plxdma stat: %.8X \n", plx_irq_reg,local_irq_reg,readl(DMACmdStatus(device)));
	//writel(0xaaaa, (Mailbox0(device)));

	//GSC16AI32_DisableIrqPlx(pDeviceInfo);
	GSC16AI32_writePlx(pDeviceInfo,INT_CTRL_STATUS,plx_irq_reg);
	/* check for DMA complete interrupt */
	if (pDeviceInfo->bDmaInProgress) 
	{
		dma_reg=GSC16AI32_readPlx(pDeviceInfo,DMA_CMD_STATUS);
		//printk(KERN_INFO DEVICE_NAME "DMA in progress. %.8lX\n",dma_reg);
		if (dma_reg & DMA0_DONE) 
		{
			//writel(0xbbbb, (Mailbox0(device)));
			/* acq interrupt */
			GSC16AI32_writePlx(pDeviceInfo, DMA_CMD_STATUS, DMA0_CLR_INT);
			/* wake up process blocked in 'read()' */
			pDeviceInfo->bDmaInProgress = FALSE;
			RtSetEvent(device->hEventDmaPending);
			//printf("dma Interrupt \n");
			//printk(KERN_INFO DEVICE_NAME "DMA end return plx irq %.8X local irq %.8X. plxdma stat%.8X.\n",readl(IntCntrlStat(device)),readlocal(device,INT_CTRL_REG),readl(DMACmdStatus(device)));
			//writel(0xdddd, (Mailbox0(device)));
			return TRUE;
		}
	}
	/*
	*	handle group 0 interrupts.
	*/
	//printk(KERN_INFO DEVICE_NAME "before Group 0 interrupt, local irq reg %.8lx\n",local_irq_reg);

	if (local_irq_reg&ICR_GROUP_0_REQUEST)
	{
		irq_type=local_irq_reg&ICR_GROUP_0_MASK;
		//writelocal(device,local_irq_reg & (~(IRQ_0_REQUEST|IRQ_0_MASK)), INT_CTRL_REG);
		GSC16AI32_andLocal(pDeviceInfo, INT_CTRL_REG,(~(ICR_GROUP_0_REQUEST|ICR_GROUP_0_MASK)));
		//printk(KERN_INFO DEVICE_NAME "Group 0 interrupt. type: %x\n",irq_type);

		switch(irq_type) 
		{
			case ICR_GROUP_0_IDLE:
			//printf("group 0 IRQ_0_IDLE Interrupt %x\n",irq_type);
			if (pDeviceInfo->bInitPending) 
			{
				pDeviceInfo->bInitPending = FALSE;
				RtSetEvent(device->hEventInitComplete);
			}
			break;
		case ICR_GROUP_0_AUTOCAL_COMPLETE:
			//printf("group 0 IRQ_0_AUTOCAL_COMPLETE Interrupt %x\n",irq_type);
			if (pDeviceInfo->bAutocalPending) 
			{
				pDeviceInfo->bAutocalPending = FALSE;
				RtSetEvent(device->hEventAutocalComplete);
			}
			break;
		case ICR_GROUP_0_INPUT_SCAN_INIT:
			//printf("group 0 IRQ_0_INPUT_SCAN_INIT Interrupt %x\n",irq_type);
			if (pDeviceInfo->bInitPending) 
			{
				pDeviceInfo->bInitPending = FALSE;
				RtSetEvent(device->hEventInitComplete);
			}
			break;
		case ICR_GROUP_0_INPUT_SCAN_COMPLETE:
			//printf("group 0 ICR_GROUP_0_INPUT_SCAN_COMPLETE Interrupt %x\n",irq_type);
			if (pDeviceInfo->bChReadyPending) 
			{
				pDeviceInfo->bChReadyPending = FALSE;
				RtSetEvent(device->hEventChannelReady);
			}
			break;
		case ICR_GROUP_0_TRIGGER_BURST_INITIALIZED:
			//printf("group 0 ICR_GROUP_0_TRIGGER_BURST_INITIALIZED Interrupt %x\n",irq_type);
			RtSetEvent(device->hEventTriggerBurstInitialized);
			GSC16AI32_orLocal(pDeviceInfo, INT_CTRL_REG,ICR_GROUP_0_TRIGGER_BURST_INITIALIZED);
			break;
		case ICR_GROUP_0_TRIGGER_BURST_COMPLETED:
			//printf("group 0 ICR_GROUP_0_TRIGGER_BURST_COMPLETED Interrupt %x\n",irq_type);
			RtSetEvent(device->hEventTriggerBurstCompleted);
			GSC16AI32_orLocal(pDeviceInfo, INT_CTRL_REG,ICR_GROUP_0_TRIGGER_BURST_COMPLETED);
			break;
		default:
			RtPrintf("ERROR - unknown group 0 Interrupt\n");
			break;
		}
	}
	/*
	*	handle group 1 interrupts.
	*/
	if (local_irq_reg&ICR_GROUP_1_REQUEST)
	{
		//printk(KERN_INFO DEVICE_NAME "Group 1 interrupt.\n");
	//	GSC16AI32_writelocal(device,local_irq_reg & (~(IRQ_1_REQUEST|IRQ_1_MASK)), INT_CTRL_REG);
		GSC16AI32_andLocal(pDeviceInfo, INT_CTRL_REG,(~(ICR_GROUP_1_REQUEST|(ICR_GROUP_1_MASK<<ICR_GROUP_1_SHIFT))));
		irq_type=(local_irq_reg >>ICR_GROUP_1_SHIFT) &ICR_GROUP_1_MASK;

		switch(irq_type) 
		{
		case ICR_GROUP_1_IDLE:
			break;
		case ICR_GROUP_1_LO_HI:
			//printf("IRQ_1_LO_HI 1 Interrupt %x\n",irq_type);
			if (pDeviceInfo->bLoHiPending) 
			{
				//device->bLoHiPending = FALSE;
				RtSetEvent(device->hEventBufferThresLowToHigh);
				GSC16AI32_orLocal(pDeviceInfo, INT_CTRL_REG,ICR_GROUP_1_LO_HI<<ICR_GROUP_1_SHIFT);
			//	nbmots = GSC16AI32_readLocal(pDeviceInfo,BUFFER_SIZE_REG);
			//	printf("nbmots = %x",nbmots);
			} 
			break;
		case ICR_GROUP_1_HI_LO:
			//printf("IRQ_1_HI_LO 1 Interrupt %x\n",irq_type);
			if (pDeviceInfo->bHiLoPending) 
			{
				device->bHiLoPending = FALSE;
				RtSetEvent(device->hEventBufferThresHighToLow);
				GSC16AI32_orLocal(pDeviceInfo, INT_CTRL_REG,ICR_GROUP_1_HI_LO<<ICR_GROUP_1_SHIFT);
			} 
			break;
		default:
			RtPrintf("ERROR - unknown group 1 Interrupt\n");
			break;
		}
	}
	/*
	*	check every possible interrupt source, to ensure that the
	* current interrupt is turned off, expected or not.
	*/
	/*
	printk(KERN_INFO DEVICE_NAME "spurious check.\n");

	// PLX interrupt-control registers.
	CLEAR_SPURIOUS_PLX(plx_irq_reg,IRQ_LOCAL_LSERR_ABORT, "IRQ_LOCAL_LSERR_ABORT*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_LSERR_ABORT, "IRQ_LOCAL_LSERR_ABORT*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_LSERR_OVERFLOW, "IRQ_LOCAL_LSERR_OVERFLOW*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_GENERATE_SERR, "IRQ_GENERATE_SERR*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_MAILBOX_ENABLE, "IRQ_MAILBOX_ENABLE*");
	//	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_PCI_ENABLE, "IRQ_PCI_ENABLE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_PCI_DOORBELL_ENABLE, "IRQ_PCI_DOORBELL_ENABLE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_ABORT_ENABLE, "IRQ_ABORT_ENABLE*");
	//	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_PCI_ENABLE, "IRQ_LOCAL_PCI_ENABLE*");
	//	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_ENABLE, "IRQ_LOCAL_ENABLE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_DOORBELL_ENABLE, "IRQ_LOCAL_DOORBELL_ENABLE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_DMA_1_ENABLE,"IRQ_DMA_1_ENABLE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_LOCAL_DOORBELL_ACTIVE,"IRQ_LOCAL_DOORBELL_ACTIVE*");
	CLEAR_SPURIOUS_PLX(plx_irq_reg, IRQ_DMA_1_ACTIVE,"IRQ_DMA_1_ACTIVE*");
	*/

	/* if something is pending, leave with the interrupt enabled. */

	if (device->bDmaInProgress || device->bAutocalPending || device->bChReadyPending || 
		device->bLoHiPending || device->bInitPending || device->bHiLoPending)
	{
		GSC16AI32_orPlx(pDeviceInfo,INT_CTRL_STATUS, (IRQ_PCI_ENABLE | IRQ_LOCAL_PCI_ENABLE|IRQ_DMA_0_ACTIVE));
	}


	return TRUE;
}
