/*+[$HB0]===================================================================+
  |                                                                         |
  |[$HI]--------------------------------------------------------------------
  |                                                                         
  | IDENTIFICATION     :                                                    
  | --------------                                                          
  | Project Name       : 
  | File Name          : GSC16AI32.h                                         
  | Target             : PC computer system (INTEL CPU)- Windows XP - RTX
  | Development System : Visual 7.0                        
  | Creation Date      :                                          
  |                                                                         
  |[$HD]--------------------------------------------------------------------
  |                                                                         
  | FILE DESCRIPTION   :                                                    
  | ----------------                                                        
  |                                                                         
  |                                                                         
  |[$HG]--------------------------------------------------------------------
  |                                                                         
  | GLOBAL DATA        :                                                    
  | -----------                                                             
  |                                                                         
  |[$HV]--------------------------------------------------------------------
  |                                                                         
  | VERSION MANAGEMENT :                                                    
  | ------------------                                                      
  |   Author   |  Date    |     Comments                                    
  | ----------------------------------------------------------------------  
  |	           |          | [v1.0] Creation.                                
  |            |          |                                                 
  +[$HE0]===================================================================+*/
#ifndef _GSC16AI32_H_
#define _GSC16AI32_H_
#ifdef  __cplusplus
extern "C" {
#endif


/*****************************************************************************/
/******                                                                 ******/
/******                         INCLUDE FILES                           ******/
/******                                                                 ******/
/*****************************************************************************/

/*****************************************************************************/
/******                                                                 ******/
/******                          LOCAL DEFINE                           ******/
/******                                                                 ******/
/*****************************************************************************/
#define GSC16AI32_MAX_BOARDS 10 /* maximum number of boards that driver supports. Arbitrarily chosen,
change if you can actually support more. */

	/* ---- scan mode codes ---- */
#define INPUT_RANGE_2_5 0
#define INPUT_RANGE_5   1
#define INPUT_RANGE_10  3
/* ---- mode config codes ---- */
#define INPUT_MODE_SHIFT	0
#define INPUT_MODE_MASK		0x07

#define INPUT_DIFFERENTIAL  0
#define INPUT_SINGLE_ENDED  1
#define INPUT_ZERO_TEST     2
#define INPUT_VREF_TEST     3
#define INPUT_SYSTEM_ANALOG_SS 0

#define SS_SINGLE_ENDED			0
#define SS_PSEUDO_DIFFERENTIAL	1
#define SS_FULL_DIFFERENTIAL	3

#define SS_MODE_SHIFT 8
#define SS_MODE_MASK (3 << 8)
/* ---- device test codes ---- */
#define FORMAT_TWO_COMPLEMENT     0
#define FORMAT_OFFSET_BINARY      1
/* ---- scan size codes ---- */
#define SCAN_SINGLE_CHAN	0
#define SCAN_2_CHAN         1
#define SCAN_4_CHAN		    2
#define SCAN_8_CHAN		    3
#define SCAN_16_CHAN		4
#define SCAN_32_CHAN		5

#define EXTERNAL_SYNCH	0
#define INTERNAL_RATE_A	1
#define INTERNAL_RATE_B	2
#define BCR_INPUT_SYNCH	3

#define MASTER_CLOCK     0
#define RATE_A_OUTPUT    1

#define BIPOLAR_INPUTS  0
#define UNIPOLAR_INPUTS BCR_UNIPOLAR_INPUTS_SSA

#define SINGLE_ENDED_PROC       BCR_DIFF_PROC_SE_SSA                
#define PSEUDO_DIFFERETIAL_PROC BCR_DIFF_PROC_PSEUDO_DIFF_SSA       
#define FULL_DIFFERENTIAL_PROC  BCR_DIFF_PROC_FULL_DIFF_SSA

// time tag sample clock source
#define TTC_INTERNAL_RATE_A 0
#define TTC_EXTERNAL_SAMP_CLK 1
#define TTC_EXTERNAL_REF_CLK 2
// time tag source reset
#define TTC_INTERNAL_RESET 1
#define TTC_EXTERNAL_RESET 2
// time tag clock source
#define TTC_INTERNAL 0
#define TTC_EXTERNAL 1
#define TRIGGER_BURST_COMPLETED 1
#define THRES_LO_HI 2

#ifndef GSC16AI32_BASE_ERROR
#define GSC16AI32_BASE_ERROR -67000
#endif

enum {
	DMA_DISABLE,
	DMA_ENABLE,
	DMA_DEMAND_MODE
};
typedef enum _E_GSC16AI32_ERROR_CODE
{
    GSC16AI32_ERR_NULL_POINTER  = GSC16AI32_BASE_ERROR,
    GSC16AI32_ERR_INVALID_PARAMETER ,
    GSC16AI32_ERR_INVALID_HANDLE ,
    GSC16AI32_ERR_MAPPING_MEMORY ,
    GSC16AI32_ERR_UNMAPING_MEMORY ,
    GSC16AI32_ERR_FAILED ,
    GSC16AI32_ERR_ABANDONED ,
    GSC16AI32_ERR_TIMEOUT ,
    GSC16AI32_ERR_AUTOCAL_FAILED ,
    GSC16AI32_ERR_FUNCT_NOT_IMPLEMENTED ,
    GSC16AI32_ERR_TRY_AGAIN ,
    GSC16AI32_ERR_BUFFER_OVERFLOW ,
    GSC16AI32_ERR_IO ,
    GSC16AI32_ERR_CARD_NOT_FOUND ,
		GSC16AI32_ERR_CARD_ALLREADY_INITIALIZED ,
		GSC16AI32_ERR_CREATE_EVENT,
		GSC16AI32_ERR_ATTACH_INTERRUPT,
		GSC16AI32_ERR_ALLOC_DMA_BUFFER ,
		GSC16AI32_ERR_UNKNOWN
} E_GSC16AI32_ERROR_CODE;
/*****************************************************************************/
/******                                                                 ******/
/******                       FUNCTIONS PROTOTYPES                      ******/
/******                                                                 ******/
/*****************************************************************************/

//==============================================================================
//
//	Initializes the board, i.e. reset
//
//==============================================================================
int GSC16AI32_Init(int	nDeviceNumber);

//==============================================================================
//
//
//==============================================================================
int GSC16AI32_Close(int	nDeviceNumber);
//==============================================================================
//
//	Initiates Auto Calibration 
//
//==============================================================================
int GSC16AI32_Autocal(int	nDeviceNumber);

//==============================================================================
//	Read Local Register Data from the Device (32Bit)
//
//  See Users Manual for register bit definitions
//
//  Returns register value 
//==============================================================================
int GSC16AI32_ReadLocal32(int	nDeviceNumber,unsigned long usRegister,unsigned long *pulValue);

//==============================================================================
//	Write Local Register Data to the Device (32Bit)
//
//  See Users Manual for register bit definitions
//
//==============================================================================
int GSC16AI32_WriteLocal32(int	nDeviceNumber,unsigned long usRegister,unsigned long ulValue);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetDmaMode(int	nDeviceNumber,unsigned long ulValue);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputRangeConfig(int	nDeviceNumber,unsigned long ulRangeConfig);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputModeConfig(int	nDeviceNumber,unsigned long ulModeConfig);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SSModeConfig(int	nDeviceNumber,unsigned long ulModeConfig);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_InputFormatConfig(int	nDeviceNumber,unsigned long ulFormatConfig);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_StartSingleScan(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanSize(int	nDeviceNumber,unsigned long ulScanSize);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanClock(int	nDeviceNumber,unsigned long ulScanClock);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ScanClockEnable(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ScanClockDisable(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisablePciInterrupt(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetBclockSource(int	nDeviceNumber,unsigned long ulBclockSource);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SingleChanSelect(int	nDeviceNumber,unsigned long ulChannel);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanRateA(int	nDeviceNumber,unsigned long ulScanRateA);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetScanRateB(int	nDeviceNumber,unsigned long ulScanRateB);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenDisableA(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenDisableB(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenEnableA(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenEnableB(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetBufferThres(int	nDeviceNumber,unsigned long ulBufferThreshold);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ClearBuffer(int	nDeviceNumber);
/*======================================================================
 * NAME : GSC16AI32_extractDeviceNumberFromRessourceName()
 *
 * DESCRIPTION: find the device number from the ressource name
 *		
 * ARGUMENTS:
 *		szRessourceName	:	name of the ressource
 *
 * RETURN VALUE:
 *		Device Number or -1 if not found
 *
 *=====================================================================*/
int GSC16AI32_extractDeviceNumberFromRessourceName(char *szRessourceName);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_revisionQuery(int	nDeviceNumber, char szDriverRevision[255], char szFirmwareRevision[255] );

/*======================================================================
 * NAME : GSC16AI32_ReadChannel()
 *
 * DESCRIPTION: 
 *		
 * ARGUMENTS:
 *
 * RETURN VALUE:
 *
 *=====================================================================*/
int GSC16AI32_ReadChannel(int	nDeviceNumber,int nChannelNumber, unsigned short * pusChannelValue);
/*======================================================================
 * NAME : GSC16AI32_ReadBuffer()
 *
 * DESCRIPTION: 
 *		
 * ARGUMENTS:
 *
 * RETURN VALUE:
 *
 *=====================================================================*/
size_t GSC16AI32_ReadBuffer(int	nDeviceNumber, char *buf, size_t size);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTag(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTag(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagConfScanClock(int	nDeviceNumber,unsigned long ulConfScanClock);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenTimeTagEnable(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GenTimeTagDisable(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagConf(int	nDeviceNumber,unsigned long ulConfClock);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ResetTimeTag(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_ResetTimeTagConf(int	nDeviceNumber,unsigned long ulResetTimeTagConf);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTagHeader(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTagHeader(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetChannelThreshold(int	nDeviceNumber,int nChannelNumber, unsigned short usValue);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetChannelReference(int	nDeviceNumber,int nChannelNumber, unsigned short usValue);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTriggerMode(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTriggerMode(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableGlobalTriggering(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableGlobalTriggering(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableUpdateRefVal(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableUpdateRefVal(int	nDeviceNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_EnableTimeTagChannel(int	nDeviceNumber,int nChannelNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_DisableTimeTagChannel(int	nDeviceNumber,int nChannelNumber);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagActiveChannelMask(int	nDeviceNumber,unsigned long ulActiveChannelMask);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_SetTimeTagBurstSize(int	nDeviceNumber,unsigned long ulBurstSize);
//==============================================================================
//
//
//==============================================================================
int GSC16AI32_GetEventHandle(int	nDeviceNumber,int nEventType,HANDLE *phEvent);
#ifdef  __cplusplus
}
#endif
#endif
